require 'puppet_readme_generator/version'
require 'puppet-strings'
require 'json'

module PuppetReadmeGenerator
  class Generator
    def initialize
      patterns ||= PuppetStrings::DEFAULT_SEARCH_PATTERNS
      options = {}
      options[:json] = 'documentation.json'
      PuppetStrings.generate(patterns, options)
    end

    def markdown
      output = []
      output << "# #{puppet_module}\n"
      output << "#### Table of Contents\n"

      count = 1

      unless long_description.nil?
        output << "#{count}. [Description](#description)"
        count += 1
      end

      if classes.length > 0
        output << "#{count}. [Classes](#classes)"
        count += 1
      end

      if defined_types.length > 0
        output << "#{count}. [Defined Types](#defined-types)"
        count += 1
      end

      unless long_description.nil?
        output << '## Description'
        output << long_description
        output << ''
      end

      if classes.length > 0
        output << '## Classes'
        classes.each do |c|
          output << ''
          output << c.markdown
        end
      end

      if defined_types.length > 0
        output << '## DefinedTypes'
        defined_types.each do |dt|
          output << ''
          output << dt.markdown
        end
      end

      output.join("\n")
    end

    def documentation_path
      'documentation.json'
    end

    def documentation
      @documentation ||= JSON.parse(File.open(documentation_path).read)
    end

    def metadata_path
      'metadata.json'
    end

    def metadata
      @metadata ||= JSON.parse(File.open(metadata_path).read)
    end

    def puppet_module
      metadata['name'].split('-')[1]
    end

    def long_description
      metadata['long_description']
    end

    def main_class
      classes.each do |c|
        return c if c.name == puppet_module
      end
      nil
    end

    def classes
      if @classes.nil?
        @classes = documentation['puppet_classes'].map do |c|
          Class.new c
        end
      end
      @classes
    end

    def defined_types
      if @defined_types.nil?
        @defined_types = documentation['defined_types'].map do |c|
          DefinedType.new c
        end
      end
      @defined_types
    end
  end

  class ClassAbstract
    def initialize(c)
      @c = c
    end

    def name
      @c['name']
    end

    def text
      @c['docstring']['text']
    end

    def examples
      if @examples.nil?
        @examples = []
        begin
          @c['docstring']['tags'].each do |t|
            next unless t['tag_name'] == 'example'
            @examples << Example.new(t, self)
          end
        rescue NoMethodError
        end
      end
      @examples
    end

    def params
      if @params.nil?
        @params = []
        begin
          @c['docstring']['tags'].each do |t|
            next unless t['tag_name'] == 'param'
            @params << Param.new(t, self)
          end
        rescue NoMethodError
        end
      end
      @params
    end

    def markdown
      output = []
      output << "### `#{@c['name']}`\n"
      output << text
      output << ''

      if params.length > 0
        output << "#### Parameters\n"
        params.each do |p|
          output << p.markdown
        end
      end

      if examples.length > 0
        output << "#### Examples\n"
        examples.each do |e|
          output << e.markdown
        end
      end

      output.join("\n")
    end

    def defaults
      @c['defaults']
    end
  end

  class Class < ClassAbstract
  end

  class DefinedType < ClassAbstract
  end

  class Param
    def initialize(t, p)
      @t = t
      @p = p
    end

    def markdown
      output = []
      output << "##### `#{@t['name']}`\n"

      if not @t['text'].nil? and @t['text'].length > 0
        output << "* #{@t['text']}"
      end

      ptype = @t['types']
      if not ptype.nil? and ptype.length > 1
        output << "* Type: `#{ptype}`"
      elsif not ptype.nil? and ptype.length == 1
        output << "* Type: `#{ptype.first}`"
      end
      default = @p.defaults[@t['name']]
      output << "* Default: `#{default}`" unless default.nil?
      output << ''
      output.join("\n")
    end
  end

  class Example
    def initialize(t, p)
      @t = t
      @p = p
    end

    def markdown
      output = []
      output << "##### #{@t['name']}\n"
      output << "```\n#{@t['text']}\n```"
      output.join("\n")
    end
  end
end
